/*
    Copyright (C) 1995-1998 AROS - The Amiga Research OS
    $Id: remsegment.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Remove a segment from the system list.
    Lang: english
*/
#include "dos_intern.h"
#include <proto/exec.h>

/*****************************************************************************

    NAME */
#include <dos/dosextens.h>
#include <proto/dos.h>

	AROS_LH1(LONG, RemSegment,

/*  SYNOPSIS */
	AROS_LHA(struct Segment *, seg, D1),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 131, Dos)

/*  FUNCTION
	Remove the segment seg from the DOS resident command list.

	The segment to be removed should be in the list, and should
	have a usercount of 0. System or internal segment cannot be
	removed (although they can be replaced).

	New: the same seglist can now appear several times in the
	list with different names. If this occurs, the Segment is
	freed but the seglist is not unloaded.

    INPUTS
	seg		- Segment to remove.

    RESULT
	!= 0	Segment was removed
	== 0	Segment was not removed (not in list, or not free).

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO
	AddSegment(), FindSegment()

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

#ifdef AROS_DOS_COMPATIBLE
    struct DosInfo *di;
#endif

    FAILFORTASKNP(return DOSFALSE;)

#ifdef AROS_DOS_COMPATIBLE
    di = BADDR(DOSBase->dl_Root->rn_Info);

    if (!di)
    {
	return DOSFALSE;
    }
#endif /* AROS_DOS_COMPATIBLE */

    /* Make sure segment is freeable */
    if(( seg->seg_UC >= 0 && seg->seg_UC <= 1 ) || seg->seg_UC == CMD_NOTLOADED)
    {
	struct Segment *next, *prev;
	BPTR seglist = seg->seg_Seg;
	BOOL found = FALSE;

	prev = NULL;
	Forbid();

#ifdef AROS_DOS_COMPATIBLE

	next = BADDR(di->di_NetHand);

#else /* AROS_DOS_COMPATIBLE */

	next = BADDR(DOSBase->dl_ResList);

#endif /* AROS_DOS_COMPATIBLE */

	while (next)
	{
	    if (next == seg)
	    {
		if (prev)
		{
		    prev->seg_Next = next->seg_Next;
		}
		else
		{
#ifdef AROS_DOS_COMPATIBLE

		    di->di_NetHand = (APTR) next->seg_Next;

#else /* AROS_DOS_COMPATIBLE */

		    DOSBase->dl_ResList = next->seg_Next;

#endif /* AROS_DOS_COMPATIBLE */
		}
		found = TRUE;
	    }
	    else if (next->seg_Seg == seglist)
	    {
		seglist = 0;
	    }

	    prev = next;
	    next = BADDR(next->seg_Next);
	}
	Permit();

	if (found)
	{
	    if (seg->seg_UC == CMD_NOTLOADED)
	    {
		FreeVec((APTR)BADDR(seglist));
	    }
	    else
	    {
		UnLoadSeg(seglist);
	    }

	    dosFreeVec(seg);

	    return DOSTRUE;
	}
    }
    else
    {
	SetIoErr(ERROR_OBJECT_IN_USE);
    }

    return DOSFALSE;

    AROS_LIBFUNC_EXIT
} /* RemSegment */
